C*****************************************
       program Graph GBCDs
C*****************************************
       implicit none

       include 'common.fi'

       !parameter (CD=9)
       !parameter (CD2=9)
       !dimension gbd(CD,CD,CD,CD2,4*CD2)
	   dimension gbd(12,12,12,12,48)
       dimension e1(3), e2(3)
       dimension gg1(3,3), gg2(3,3), dg(3,3)
       dimension so1(3,3), so2(3,3), sa(2), so2t(3,3)
       dimension ax(3)

       integer point
       real minmrd, interval

       logical first
       integer xjk,i1,i2

!  CODE::
       write(6,*) '=============================================='
       write(6,*) ' PROGRAM TO PRODUCE STEREOGRAPHIC PROJECTIONS'
       write(6,*) ' OF GRAIN BOUNDARY PLANE DISTRIBUTIONS FROM'
       write(6,*) ' GRAIN BOUNDARY CHARACTER DISTRIBUTIONS '
       write(6,*)
       write(6,*) ' DEVELOPED BY THE CMU MRSEC'
       write(6,*) '  version 03/16/2010'
       write(6,*) '  rohrer@cmu.edu'	   
       write(6,*) '=============================================='
       
	   version = '03/16/2010'	  	   
	   
       degrad = 180./3.14159
       pi = 4.0*atan(1.0)
       eps = 1.e-6   !  used for testing angles
       HCD = 90.0

       open(21, file='input.txt', status='old')
       read(21,*) keyword
       point = index(keyword,'.') - 1
       read(21,*) tempword
       read(21,*) msym
	   read(21,*) tempword
       read(21,*) plot_type
	   read(21,*) tempword
       read(21,*) normal_dir
	   read(21,*) tempword
       read(21,*) CD, CD2
	   	   
  5    format(A,A)
  6    format(A)
	   write(6,5)'I am graphing data in the file labeled: ', keyword
	   if (plot_type.eq.1) then 
	       write(6,6)'The data will be in the format for gmt'	   
	   endif
	   if (plot_type.eq.2) then 
	       write(6,6)'The data will be in the format for 3D field'	   
	   endif
	   if (plot_type.eq.3) then 
	       write(6,6)'Data will be output in both the gmt and 3D field formats'	   
	   endif
	   if (msym.eq.3) then 
	       write(6,6)'you have specified the symmetry as cubic'	   
	   endif
	   if (msym.eq.2) then 
	       write(6,6)'you have specified the symmetry as hexagonal'	   
	   endif	   
	   if (msym.eq.1) then 
	       write(6,6)'you have specified the symmetry as tetragonal'	   
	   endif	   
	   if (msym.eq.4) then 
	       write(6,6)'you have specified the symmetry as trigonal'	   
	   endif	   
       
c      this is the section where we set the viewpoint for the projection

c      for a [001] projection, there is no rotation of the data
	   if (normal_dir.eq.1) then 
		   ax_ref(1)=0.0
	       ax_ref(2)=0.0
	       ax_ref(3)=1.0
	       ang_ref= 0.0
	       write(6,6)'The [001] direction will be normal to the plane of the drawing.'	   
	   endif

c      for a [110] projection, rotate 90 deg around [1-10]
	   if (normal_dir.eq.2) then 
	       ax_ref(1)=0.7071
	       ax_ref(2)=-0.7071
	       ax_ref(3)=0.0
           ang_ref= pi*(90.0/180.0)
	       write(6,6)'The [110] direction will be normal to the plane of the drawing.'			   
	   endif

c      for a [111] projection, rotate 54.7 deg around [1-10]
	   if (normal_dir.eq.3) then
	       ax_ref(1)=0.7071
	       ax_ref(2)=-0.7071
	       ax_ref(3)=0.0
           ang_ref= pi*(54.7/180.0)
	       write(6,6)'The [111] direction will be normal to the plane of the drawing.'			   
	   endif	   

c      for a [100] projection, rotate 90 deg around [0-10]
	   if (normal_dir.eq.4) then 
	       ax_ref(1)=0.0
	       ax_ref(2)=-1.0
	       ax_ref(3)=0.0
           ang_ref= pi*(90.0/180.0)
	       write(6,6)'The [100] direction will be normal to the plane of the drawing.'			   
	   endif	
	   write(6,*) ' '
c      this is the matrix that will rotate the vectors on the plot.
	   call AAToG (ax_ref, ang_ref, g_ref)
	   call trans (g_ref, g_ref_t)
	   
c  The statement calls the subroutine that contains the symmetry operators.  The variable
c  msym, read from the input file, specifies the symmetry (1 = tetragonal, 2 = hexagonal, 
c  3 = subic, 4 = trigonal), O is a matrix containing the operators, and nsymm is the 
c  number of symmetry operators.  The routine returns O and nsymm.

       call get_symop (msym, O, nsymm)

c  This part is to zero the grain boundary character distribution matrix		
       do i=1,CD
          do j=1,CD
             do k=1,CD
                do ii=1,CD2
                   do jj=1,4*CD2
                      gbd(i,j,k,ii,jj)=0.0
                   enddo
                enddo
             enddo
          enddo
       enddo	

c Check each line for the string '#'. If found, we know this line is part  
c of the header.  If not, we have reached the data section.		
	   open(33, file=keyword, status='old')
       header = 0
       hash = 0
	   do i=1,100  ! max header length is 100 lines
	    read(33, "(a)") inline
	    hash = index(inline, '#')
	    if (hash.ne.0) then
	     header=header+1
		 hash=0
	    else
		 goto 8     
	    endif			
	   enddo
 8    close(33)		









c  the data is read from the file specified in the input file into matrix gbd

       open (33, file=keyword, status='old')
		do i=1,header
		 read(33, "(a)") inline
		enddo			 
       do i=1,CD
          do j=1,CD
             do k=1,CD
                do ii=1,CD2
                   do jj=1,4*CD2
                      read (33,*) gbd(i,j,k,ii,jj)
                   enddo
                enddo
             enddo
          enddo
       enddo
       close (33)
	   
c      the number of plots to be produced is determined
       
       read (21,*) tempword
       read (21,*) numplots
       if (numplots.ge.10) then
          write (6,*) 'at most, I make 10 plots'
       endif
 10    format(A,i1,A)
       write (6,10) 'I am planning to make ',numplots,' plots'
       read (21,*)tempword9, tempword9, tempword9, tempword9
       plot = 0
       do plot=1,numplots
c       the axis and angle are read from the input file

          first = .true.

 20       format(A,i1)
          write(6,20) 'plot number= ',plot	    
          read (21,*) ax(1), ax(2), ax(3), ang
 30       format (A,f5.2,2X,f5.2,2X,f5.2,2X,f6.2,2X)		
          write (6,30) 'now plotting: ',ax(1), ax(2), ax(3), ang
          sum=0.0
          ct=0

          one=char(48+plot)
		
c       here, two files are opened into which the results will be written
c       the first (34) in 3D field format.  The second in GMT format.		
		
 	      if (plot_type.eq.2.or.plot_type.eq.3) then
           open (34, file=keyword(1:point)//one//'.dat'
     &         , status='unknown')
	      endif
 	      if (plot_type.eq.1.or.plot_type.eq.3) then		  
          open (35, file=keyword(1:point)//'_gmt_'//one//'.dat'
     &         ,status='unknown')
          write(35,"(4(1x,f4.1))")  ax(1), ax(2), ax(3), ang 
	      endif
c       the axis and angle, input as miller indices and degrees of rotation, 
c       is converted to a unit vector and rotation in radians

          ss=sqrt(ax(1)*ax(1)+ax(2)*ax(2)+ax(3)*ax(3))
          ax(1)=ax(1)/ss
          ax(2)=ax(2)/ss
          ax(3)=ax(3)/ss
          ang=ang*(pi/180.0)

c       this converts the axis and angle values to a g matrix
		
          call AAToG (ax, ang, g)

c       i_plot is the azimuthal angle.  It takes 30 discrete values between
c       0 and 90, assigned to theta_plot.  j_plot is the in plane angle and
c       it takes 120 values in range between 0 and 360, assigned to phi_plot.
		
	        summrd=0.0
            maxmrd = 0.0
            minmrd = 1.e+36  ! large value
	        do i_plot=1,31
               do j_plot=1,121

		        theta_plot=3.0*float(i_plot-1)*(pi/180.0)
			    phi_plot=3.0*float(j_plot-1)*(pi/180.0)		  
c               this call takes angle theta and phi and convert them to a vector, n_plot  
                call AnglesToV (theta_plot, phi_plot, n_plot_1)
c               this rotates the reference frame.  Without this, it is a [001] projection
				call MToV (g_ref_t, n_plot_1, n_plot) 
c               this assigns the normal of interest to n1
				sum=0.0
				n1(1)=n_plot(1)
				n1(2)=n_plot(2)
				n1(3)=n_plot(3)
c		        this call returns the transpose of g
				call trans (g, gt)
c		        this call rotates n by the transposed matrix, and names it n2
                call MToV (gt, n1, n2)
				ct=0
		  
c               in this next loop, n1 and n2 are hit with all the symmetry operators, then
c               the bin corresponding to each vector is queried.  Values read from all of
c               indistinguishable bins are then averaged to yeild a value for this 
c               this specific orientation.
		  
		        do ii_plot=1,nsymm
				  do jj_plot=1,nsymm	 
				  call symop (O, ii_plot, so1)
c                 this rotates n1 by the first, non-transposed operator
				  call MToV (so1, n1, nf)  
c                 get the second symmetry operator
                  call symop (O, jj_plot, so2)
c                 this returns the transpose of the second symmetry operator
				  call trans (so2,so2t)
c		          these steps determine the misorientation, then find the
c                 equivalent Euler angles
				  call MToM (g, so2t, gg1)
				  call MToM (so1, gg1, dg)
				  call GToE (dg, mort)
c		          We only consider those misorientations in the domain where
c                 phi1, cap_phi, and phi2 are between zero and 90
			      if (mort(1).lt.(pi*0.5+eps) .and. 
     &                       mort(2).lt.(pi*0.5+eps) .and. 
     &                       mort(3).lt.(pi*0.5+eps) ) then
c                  this returns the azimuthal (0 to 90) and in-plane (0 to 360) angles	 
	               call VToAngles (nf, sa)
c                  these steps find the bin that corresponds to five parameters (mort(3) and sa(2))			
				   c1=int(float(CD)*2.0*mort(1)/pi)+1
				   c2=int(float(CD)*cos(mort(2)))+1
				   c3=int(float(CD)*2.0*mort(3)/pi)+1
				   c4=int(float(CD2)*cos(sa(1)))+1
				   c5=int(float(4*CD2)*sa(2)/(pi*2.0))+1
				   if (c1.eq.CD+1) c1=CD
				   if (c2.eq.CD+1) c2=CD
				   if (c3.eq.CD+1) c3=CD
				   if (c4.eq.CD2+1) c4=CD2
				   if (c4.eq.0) c4=1
				   if (c5.eq.4*CD2+1) c5=4*CD2
c			       here, we add the value in this bin to a total and increment the
c                  counter (ct) so it can be properly averaged later
				   sum=sum+gbd(c1,c2,c3,c4,c5)
				   ct=ct+1
			      endif
			
c                 the next section of code repeats what is done above, except that
c                 the crystals have been exchanged, so we are considering the interface
c                 orientation in the reference frame of the second crystal			
			
				  call MToV (so1, n2, nf)
				  call trans (g, gt)
				  call MToM (gt, so2, gg2)
				  call MToM (so1, gg2, dg)
				  call GToE (dg, mort)
			      if (mort(1).lt.(pi*0.5+eps) .and. 
     &                       mort(2).lt.(pi*0.5+eps) .and. 
     &                       mort(3).lt.(pi*0.5+eps) ) then
				   call VToAngles (nf, sa)
				   c1=int(float(CD)*2.0*mort(1)/pi)+1
				   c2=int(float(CD)*cos(mort(2)))+1
				   c3=int(float(CD)*2.0*mort(3)/pi)+1
				   c4=int(float(CD2)*cos(sa(1)))+1
				   c5=int(float(4*CD2)*sa(2)/(pi*2.0))+1
				   if (c1.eq.CD+1) c1=CD
				   if (c2.eq.CD+1) c2=CD
				   if (c3.eq.CD+1) c3=CD
				   if (c4.eq.CD2+1) c4=CD2
				   if (c4.eq.0) c4=1
				   if (c5.eq.4*CD2+1) c5=4*CD2
				   sum=sum+gbd(c1,c2,c3,c4,c5)
				   ct=ct+1
			      endif
				 enddo   !jj_plot loop
		        enddo   !ii_plot loop
         
c        this projects the angles for the stereographic projection		 
		 
                 xp=tan(theta_plot*0.5)*cos(phi_plot)
                 yp=tan(theta_plot*0.5)*sin(phi_plot)
		 
c        this line computes the average from all of the indistinuishable bins		 
		 
                 amrd=sum/float(ct)

 750             format(f8.5,3x,f8.5,3x,f8.3,2(i4))
 
c        Write values for ploting in 3D field
 	   if (plot_type.eq.2.or.plot_type.eq.3) then
	     write (34,750) xp, yp, amrd, i_plot, j_plot
		endif 
c        Write values for ploting by GMT		 
	   if (plot_type.eq.1.or.plot_type.eq.3) then
		 write (35,*) phi_plot*degrad, (90.-(theta_plot*degrad)), amrd
		endif
		 summrd = summrd + amrd
		 if (maxmrd.lt.amrd) maxmrd = amrd
		 if (minmrd.gt.amrd) minmrd = amrd
		 
              first = .false.
              enddo
	   enddo
 760   format(A,f5.2,A)	
 770   format(A,f7.2,A)    
       write(6,760) 'For this misorientation, the average value is '
     &      ,summrd/3600,' MRD.'
       write(6,770) 'For this misorientation, the maximum value is '
     &      ,maxmrd,' MRD.'
       write(6,770) 'For this misorientation, the minimum value is '
     &      ,minmrd,' MRD.'
	   
c       close(34)
       close(35)
	   
      enddo
      close (21)
	   
	  interval=(maxmrd-minmrd)/10
	   
  780  format(A,I1,1x,A,A,f5.2,1x,f6.2,1x,f5.2)	   
	   if (plot_type.eq.1.or.plot_type.eq.3) then 
	       write(6,*) ' '
	       write(6,6)'to initiate the gmt script, enter ./Draw_stereograms [Number of plots] [filename]_gmt_ 5d rainbow [min] [max] [interval]'
		   write(6,780)'For example: ./Draw_stereograms ',numplots,keyword(1:point),'_gmt_ 5d rainbow ',minmrd,maxmrd,interval
	   endif	   
	   
	   
	   
      continue	   
      end
